if not (yakuzaBossTexts) then
	rawset(_G, "yakuzaBossTexts", {})
end

local currentBoss = nil
local textScale = 20*FRACUNIT
local textTimer = 0
local zoomSpeed = (FRACUNIT/5)*4
local ignoreBosses = {}

local function ResetText()
	textScale = 20*FRACUNIT
	textTimer = 0
end

local function YakuzaMapLoad()
	ResetText()
	ignoreBosses = {}
end

addHook("MapLoad", YakuzaMapLoad)

local ENDTICS = 1+TICRATE+(TICRATE/2)

local function DoTextScaling()

	if not (leveltime > 1*TICRATE)
	or not (consoleplayer) then return end

	if (textTimer < ENDTICS-1) then
		if (textScale > FRACUNIT) then
			textScale = $1-zoomSpeed
		else
			textTimer = $1+1
		end
	else
		if (textScale < 20*FRACUNIT) then
			textScale = $1+($1/2)
		else
			textTimer = ENDTICS
		end
	end
	
	if (textScale < FRACUNIT) then
		textScale = FRACUNIT
	end


end

addHook("ThinkFrame", DoTextScaling)


local function YakuzaBossCheck(mo)
	for player in players.iterate do
		if not (player.mo.valid)
		or not (yakuzaBossTexts[mo.type]) then return end
		
		if (P_CheckSight(mo, player.mo)) then
			//already showing text
			if (textTimer)
			and not (textTimer > ENDTICS) then
				return
			end
			
			//already displayed text this map
			if (ignoreBosses[mo.type]) then
				return
			end
			
			//already targetted this tic
			if (player.mo.yakuzaboss)
			and not (textTimer) then
				return
			end
			
			//delay fang until he pops out
			if (mo.type == MT_FANG)
			and (mo.state >= S_FANG_SETUP)
			and (mo.state <= S_FANG_INTRO5)
				return
			end

			if player == displayplayer
				ignoreBosses[mo.type] = true
				ResetText()
			end
			player.mo.yakuzaboss = mo.type
			//set as targetted enemy for boss
			//doesn't look that good so disabled
			/*if (player.mo.kiryuinfo) then
				player.mo.kiryuinfo.enemy = mo
			end*/
		end
	end
end

addHook("BossThinker", YakuzaBossCheck)

local function BossIntro(v, player)	

	local bText = ""
	local bInfo = ""
	local hudText = {}
	local rows = 1
	hudText[rows] = {}
	local hudInfo = {}
	
	local yakuzaLetters = {}

	if not (player.mo)
	or not (player.mo.yakuzaboss)
	or not (player.mo.skin == "kiryu") then return end
	
	//freeslot the capital letters
	for i = 65, 65+25, 1 do
		local letterPatch = string.char(i)
		yakuzaLetters[i] = v.cachePatch("YAKU\$letterPatch\")
	end
	
	//add width for spaces
	yakuzaLetters[32] = {width = 15}
	
	local extraSymbols = {
		"%", "&", "(", ")", ".", ",", "-", "_", "^", "=", "+", "1", "2", "3", "4", "5", "6", "7", "8", "9", "0", "!", "?", "'"
	}

	for _,letterPatch in ipairs(extraSymbols) do
		local letterPatch = string.byte(letterPatch)
		yakuzaLetters[letterPatch] = v.cachePatch("YAKU\$letterPatch\")
	end


	bText = yakuzaBossTexts[player.mo.yakuzaboss].name
	bInfo = yakuzaBossTexts[player.mo.yakuzaboss].info
	
	bText, bInfo = string.upper($1), string.upper($2)
	
	local tMod = 0
	for i = 1, string.len(bText) do
		local curChar = string.sub(bText, i, i)
		//create a new line
		if (curChar == "\n") then
			rows = $1+1
			hudText[rows] = {}
			tMod = i
			
			continue
		end
		
		hudText[rows][i-tMod] = string.byte(curChar)
	end
	
	for i = 1, string.len(bInfo) do
		local curChar = string.sub(bInfo, i, i)
		hudInfo[i] = string.byte(curChar)
	end
	
	local flags = V_SNAPTOLEFT|V_SNAPTOTOP
	
	if (textScale/FRACUNIT > 1) then
		flags = $1|V_60TRANS
	end


	//get base screenwidth & height
	local screenWA, screenWB = v.dupx()
	local screenHA, screenHB = v.dupy()

	local SCREENWIDTH = (((screenWB*320)/screenWA)/FRACUNIT)
	local SCREENHEIGHT = (((screenHB*200)/screenHA)/FRACUNIT)


	if not (leveltime > 1*TICRATE) then return end
	if not (textTimer >= ENDTICS) then
		//BIG TEXT
		for row,_ in ipairs(hudText) do
			local totalWidth = 0
			//get width of current line
			for _,val in ipairs(hudText[row]) do
				totalWidth = $1 + yakuzaLetters[val].width
			end

			//BIG LETTERS
			local currentWidth = 0
			for pos, val in ipairs(hudText[row]) do
				
				if (val == 32) then 
					currentWidth = $1+15
					continue
				end
				
				
				local CHARWIDTH = yakuzaLetters[val].width
				local CHARHEIGHT = 40
				currentWidth = $1 + CHARWIDTH

				
				//get letter's offset based on the position in the word
				local xOffset = ((currentWidth)-CHARWIDTH)*textScale
				local yOffset = ((CHARHEIGHT*row)-CHARHEIGHT)*textScale

				//center the word
				xOffset = $1+(SCREENWIDTH*FRACUNIT/2)-((totalWidth*textScale)/2)
				yOffset = $1+(SCREENHEIGHT*FRACUNIT/2)-(((CHARHEIGHT*rows)*textScale)/2)
				
				v.drawScaled(xOffset, yOffset, textScale, yakuzaLetters[val], flags)
			end
		end
		
		
		local totalWidth = 0
		//get width
		for _,val in ipairs(hudInfo) do
			totalWidth = $1 + yakuzaLetters[val].width
		end
		
		//lil letters
		local currentWidth = 0
		for pos, val in ipairs(hudInfo) do
			
			if (val == 32) then 
				currentWidth = $1+15
				continue
			end
			
			
			local CHARWIDTH = yakuzaLetters[val].width
			local CHARHEIGHT = 40
			currentWidth = $1 + CHARWIDTH
			local scale = textScale/3
			
			//get letter's offset based on the position in the word
			local xOffset = ((currentWidth)-CHARWIDTH)*scale
			local yOffset = ((CHARHEIGHT)-CHARHEIGHT)*textScale

			//center the word
			xOffset = $1+(SCREENWIDTH*FRACUNIT/2)-((totalWidth*scale)/2)
			yOffset = $1+(SCREENHEIGHT*FRACUNIT/2)-(((CHARHEIGHT*rows)*textScale)/2)
			
			v.drawScaled(xOffset, yOffset-15*FRACUNIT, scale, yakuzaLetters[val], flags)
		end
		
	end
end


hud.add(BossIntro)